#!/usr/bin/python
import playerstat

__doc__ = """Boost Software License - Version 1.0 - August 17th, 2003

Permission is hereby granted, free of charge, to any person or organization
obtaining a copy of the software and accompanying documentation covered by
this license (the "Software") to use, reproduce, display, distribute,
execute, and transmit the Software, and to prepare derivative works of the
Software, and to permit third-parties to whom the Software is furnished to
do so, all subject to the following:

The copyright notices in the Software and this entire statement, including
the above license grant, this restriction and the following disclaimer,
must be included in all copies of the Software, in whole or in part, and
all derivative works of the Software, unless such copies or derivative
works are solely in the form of machine-executable object code generated by
a source language processor.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
DEALINGS IN THE SOFTWARE."""

class results():
    def __init__(self):
        self.values = { 'Bomb' : 0,
	              'R' : 0,
		      'Y' : 0,
		      'G' : 0,
		      'B' : 0,
		      'L' : 0}
        self.islilac = False

    def calcdamage(self,stat):
        alldamage = self.values['Bomb']
	for color,damage in [('R',10),('Y',6),('G',3)]:
	    if self.values[color] + stat.values[color] >= 15:
	        alldamage += damage
	return alldamage

    def __repr__(self):
        return str(self.values) + " " + str(self.islilac)

class resultseval(results):
    def __init__(self):
        results.__init__(self)
        self.value = 0.0
    def eval(self):
        self.value = self.values['Bomb'] + \
	             self.values['R'] * 2.0/3.0 + \
	             self.values['Y'] * 6.0/15.0 + \
	             self.values['G'] * 1.0/5.0 + \
	             self.values['L'] * 1.0/5.0 + \
	             self.values['B'] * 6.0/15.0 

    def __repr__(self):
        string = "Value: %f %s lilac: %s" % (self.value,str(self.values),str(self.islilac))
        return string

class resultsevalparam(resultseval):
    def __init__(self,mBomb = 1 ,mR = 1,mY = 1,mG = 1,mB = 1,mL = 1):
        resultseval.__init__(self)
        self.multipliers = { 'Bomb' : mBomb,
	              'R' : mR,
		      'Y' : mY,
		      'G' : mG,
		      'B' : mB,
		      'L' : mL}
        
    def eval(self):
        self.value = self.values['Bomb']  * self.multipliers['Bomb'] +\
	             self.values['R']     * self.multipliers['R']    +\
	             self.values['Y']     * self.multipliers['Y']    +\
	             self.values['G']     * self.multipliers['G']    +\
	             self.values['L']     * self.multipliers['L']    +\
	             self.values['B']     * self.multipliers['B']  

    def __repr__(self):
        #string = resultseval.__repr__(self)
	
	string = ''
	for type in self.values.keys():
	    if self.values[type] > 0:
	        string += "%s: %d * %0.2f " % (type,self.values[type],self.multipliers[type])
	if self.islilac == True:
            string = "Value: %0.2f { %s } (lilac )" % (self.value,string)
	else:
            string = "Value: %0.2f { %s } (normal)" % (self.value,string)
	return string

class resultsevalparamstat(resultsevalparam):
    def __init__(self,stat,mBomb = 1 ,mR = 1,mY = 1,mG = 1,mB = 1,mL = 1):
        resultsevalparam.__init__(self,mBomb,mR,mY,mG,mB,mL)
        self.stat = stat
    def eval(self):
        resultsevalparam.eval(self)
	self.value += 0.01 * self.calcdamage(self.stat)
         
class resultssplitteddamage(results):
    def __init__(self):
        results.__init__(self)

    def calcdamagevalues(self):
	lifedamage = 0
	shielddamage = 0
        damage = self.calcdamage(self.stat)
        
	if (self.ostat.values['Shield'] - damage) <= 0:
	    shielddamage = self.ostat.values['Shield']
	    lifedamage = damage - shielddamage
	else:
            shielddamage = damage

	return (shielddamage,lifedamage)
    
class resultsplayer(resultsevalparamstat,resultssplitteddamage):
    
    def __init__(self,stat,ostat,fshielddam,flifedam,mBomb = 1 ,mR = 1,mY = 1,mG = 1,mB = 1,mL = 1):
        resultsevalparamstat.__init__(self,stat,mBomb,mR,mY,mG,mB,mL)
	self.ostat = ostat
	self.fshielddam = fshielddam
	self.flifedam = flifedam


    def eval(self):
        resultsevalparamstat.eval(self)
	shieldamage,lifedamage = self.calcdamagevalues()
	self.value += self.fshielddam * shieldamage
	self.value += self.flifedam   * lifedamage

class resultsopponent(resultseval,resultssplitteddamage):
    def __init__(self,ostat,pstat,fshielddam,flifedam):
        resultseval.__init__(self)
	#for the opponent the player is the opponent
	self.ostat = pstat
	self.stat = ostat
	self.fshielddam = fshielddam
	self.flifedam = flifedam

    def eval(self):
        resultseval.eval(self)
    
	shieldamage,lifedamage = self.calcdamagevalues()
	self.value += self.fshielddam * shieldamage
	self.value += self.flifedam   * lifedamage

class resultsblue(resultsplayer):
    def eval(self):
        oldblue = self.values['B']
        neededblue = 15 - self.stat.values['Shield']
	overblue = self.values['B'] - neededblue
	if overblue > 0:
	    self.values['B'] -= overblue
	    
        resultsplayer.eval(self)

        self.values['B'] = oldblue
	    
    
